package higa.dbawt;

import java.awt.Choice;
import java.awt.Dimension;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.sql.Statement;
import java.sql.ResultSet;

import java.util.Hashtable;

import higa.sql.dataset.PickListAware;
import higa.sql.dataset.ColumnAware;
import higa.sql.dataset.IDataSet;
import higa.sql.dataset.HDatabase;
import higa.sql.dataset.Column;
import higa.sql.dataset.DataChangeEvent;
import higa.sql.dataset.DataChangeListener;
import higa.sql.dataset.NavigationEvent;
import higa.sql.dataset.NavigationListener;

public class HChoice extends Choice
  implements PickListAware, ColumnAware,
  DataChangeListener, NavigationListener,
  ItemListener {

  private String pickListTableName;
  private String pickListColumnName;
  private String pickListDisplayColumnName;
  private String columnName;
  private IDataSet dataSet;

  private Hashtable items = new Hashtable();
  private Hashtable refItems = new Hashtable();
  private static final String dummy =
    new String("");

  private transient boolean addNotifyCalled = false;
  private transient boolean setupDone = false;

  public HChoice() {
    addItemListener(this);
  }

  public void setPickListTableName(String s) {
    if (s != null && s.equals("")) {
      s = null;
    }

    pickListTableName = s;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }
  }

  public String getPickListTableName() {
    return pickListTableName;
  }

  public void setPickListColumnName(String s) {
    if (s != null && s.equals("")) {
      s = null;
    }

    pickListColumnName = s;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }
  }

  public String getPickListColumnName() {
    return pickListColumnName;
  }

  public void setPickListDisplayColumnName(String s) {
    if (s != null && s.equals("")) {
      s = null;
    }

    pickListDisplayColumnName = s;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }
  }

  public String getPickListDisplayColumnName() {
    return pickListDisplayColumnName;
  }

  public void setColumnName(String s) {
    columnName = s;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }
  }

  public String getColumnName() {
    return columnName;
  }

  public void setDataSet(IDataSet ds) {
    if (dataSet != null) {
      dataSet.removeDataChangeListener(this);
      dataSet.removeNavigationListener(this);
    }

    dataSet = ds;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }

    if (dataSet != null) {
      dataSet.addDataChangeListener(this);
      dataSet.addNavigationListener(this);
    }
  }

  public IDataSet getDataSet() {
    return dataSet;
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null) {
        openDataSet();
      }
    }
  }

  public Dimension getPreferredSize() {
    Dimension ps = super.getPreferredSize();
    if (ps == null || ps.width < 40) {
      ps = ps != null ? ps : new Dimension(100, 25);
      ps.width = ps.width > 100 ? ps.width : 100;
    }
    
    return ps;
  }

  public String getItemValue() {
    String item = getSelectedItem();
    return getItemValue(item);
  }

  public String getItemValue(int index) {
    String item = getItem(index);
    return getItemValue(item);
  }

  public String getItemValue(String item) {
    return (String)items.get(item);
  }

  public String getItemByValue(Object value) {
    if (value != null) {
      String v = value.toString();
      String s = (String)refItems.get(v);
      return s != null ? s : dummy;
    } else {
      return dummy;
    }
  }

  public void itemStateChanged(ItemEvent e) {
    postSelection();
  }

  public void dataChanged(DataChangeEvent e) {
    updateSelection();
  }

  public void navigated(NavigationEvent e) {
    if (!setupDone) {
      setupItems();
    }

    updateSelection();
  }

  public void setupItems() {
    if (pickListTableName == null) {
      if (dataSet != null) {
        Column column = dataSet.getColumn(columnName);
        pickListTableName = column.getPickListTableName();
      }
    }

    if (pickListColumnName == null) {
      if (dataSet != null) {
        Column column = dataSet.getColumn(columnName);
        pickListColumnName = column.getPickListColumnName();
      }
    }

    if (pickListDisplayColumnName == null) {
      if (dataSet != null) {
        Column column = dataSet.getColumn(columnName);
        pickListDisplayColumnName =
          column.getPickListDisplayColumnName();
      }
    }

    if (pickListTableName == null ||
      pickListColumnName == null ||
      pickListDisplayColumnName == null ||
      dataSet == null ||
      setupDone) {

      return;
    }

    HDatabase db = dataSet.getDatabase();
    if (db == null) {
      return;
    }

    removeAll();
    items.clear();
    refItems.clear();
    try {
      String query = "SELECT " +
        pickListDisplayColumnName +
        "," + pickListColumnName +
        " FROM " + pickListTableName;

      Statement stmt = db.createStatement();
      ResultSet rs = stmt.executeQuery(query);
      while (rs.next()) {
        String display = rs.getString(1);
        String value = rs.getString(2);
        addItem(display);
        items.put(display, value);
        refItems.put(value, display);
      }

      Column column = dataSet.getColumn(columnName);
      if (column.isNullable()) {
        addItem(dummy);
      }

      rs.close();
      stmt.close();
    } catch (Exception ex) {
      ex.printStackTrace();
    }

    setupDone = true;
  }

  protected void updateSelection() {
    if (dataSet == null ||
      dataSet.getColumn(columnName) == null) {

      return;
    }

    String value = dataSet.getString(columnName);
    String item = getItemByValue(value);
    select(item);
  }

  protected void postSelection() {
    String item = getSelectedItem();
    if(item == null || dataSet == null ||
      dataSet.getColumn(columnName) == null) {

      return;
    }

    try {
      String value = getItemValue(item);
      dataSet.set(columnName, value);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  private void openDataSet() {
    if (dataSet == null || !addNotifyCalled) {
      return;
    }

    if (!dataSet.isOpen()) {
      try {
        dataSet.open();
      } catch (Exception ex) {
        ex.printStackTrace();
        return;
      }
    }

    if (dataSet.isOpen() && !setupDone) {
      setupItems();
      updateSelection();
    }
  }
}