package higa.dbawt;

import java.awt.Label;

import java.text.Format;

import higa.sql.dataset.Column;
import higa.sql.dataset.DataChangeEvent;
import higa.sql.dataset.DataChangeListener;
import higa.sql.dataset.NavigationEvent;
import higa.sql.dataset.NavigationListener;
import higa.sql.dataset.ColumnAware;
import higa.sql.dataset.IDataSet;

import higa.util.Formatter;

public class HLabel extends Label
  implements ColumnAware,
  DataChangeListener,
  NavigationListener  {

  private IDataSet dataSet;
  private String columnName;
  private Format formatter;
  private String formatPattern;

  private transient boolean addNotifyCalled = false;

  public HLabel() {
  }

  public void setDataSet(IDataSet ds) {
    if (dataSet != null) {
      dataSet.removeDataChangeListener(this);
      dataSet.removeNavigationListener(this);
    }

    dataSet = ds;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }

    if (dataSet != null) {
      dataSet.addDataChangeListener(this);
      dataSet.addNavigationListener(this);
    }
  }

  public IDataSet getDataSet() {
    return dataSet;
  }

  public void setColumnName(String s) {
    columnName = s;
    if (addNotifyCalled &&
      dataSet != null) {

      openDataSet();
    }
  }

  public String getColumnName() {
    return columnName;
  }

  public void setFormatter(Format formatter) {
    this.formatter = formatter;
    formatPattern = null;
    updateText();
  }

  public Format getFormatter() {
    return formatter;
  }

  public void setFormatPattern(String pattern) {
    formatPattern =
      pattern != null && pattern.length() != 0 ?
      pattern : null;

    formatter = null;
    updateText();
  }

  public String getFormatPattern() {
    return formatPattern;
  }

  public void addNotify() {
    super.addNotify();
    if (!addNotifyCalled) {
      addNotifyCalled = true;
      if (dataSet != null) {
        openDataSet();
      }
    }
  }

  public void dataChanged(DataChangeEvent e) {
    updateText();
  }

  public void navigated(NavigationEvent e) {
    updateText();
  }

  protected void updateText() {
    if (dataSet == null) {
      return;
    }

    Column column = dataSet.getColumn(columnName);
    if (column == null) {
      return;
    }

    if (formatter == null &&
      formatPattern != null) {

      int type = column.getType();
      formatter = Formatter.getDefaultFormatter(
        type, formatPattern);
    }

    String s = "";
    if (formatter == null) {
      s = dataSet.getString(columnName);
    } else {
      Object value = dataSet.get(columnName);
      s = Formatter.format(formatter, value);
    }

    setText(s);
  }

  private void openDataSet() {
    if (dataSet == null || !addNotifyCalled) {
      return;
    }

    if (!dataSet.isOpen()) {
      try {
        dataSet.open();
      } catch (Exception ex) {
        ex.printStackTrace();
        return;
      }
    }

    if (dataSet.isOpen()) {
      updateText();
    }
  }
}