package higa.dbswing;

//import com.sun.java.swing.ButtonGroup;
import javax.swing.ButtonGroup;

import java.io.Serializable;

import higa.sql.dataset.IDataSet;
import higa.sql.dataset.ColumnAware;
import higa.sql.dataset.DataChangeEvent;
import higa.sql.dataset.DataChangeListener;
import higa.sql.dataset.NavigationEvent;
import higa.sql.dataset.NavigationListener;

import higa.util.EventMulticaster;

public class HButtonGroup extends ButtonGroup
  implements ColumnAware,
  DataChangeListener, NavigationListener,
  Serializable {

  private String columnName;
  private IDataSet dataSet;

  private EventMulticaster dataChangeCaster =
    new EventMulticaster();

  private EventMulticaster navigationCaster =
    new EventMulticaster();

  public HButtonGroup() {
  }

  public HButtonGroup(IDataSet newDataSet,
    String newColumnName) {

    setDataSet(newDataSet);
    setColumnName(newColumnName);
  }

  public void setDataSet(IDataSet newDataSet) {
    if (dataSet != null) {
      dataSet.removeDataChangeListener(this);
      dataSet.removeNavigationListener(this);
    }

    dataSet = newDataSet;
    if (dataSet != null) {
      openDataSet();
    }

    if (dataSet != null) {
      dataSet.addDataChangeListener(this);
      dataSet.addNavigationListener(this);
    }
  }

  public IDataSet getDataSet() {
    return dataSet;
  }

  public void setColumnName(String newColumnName) {
    columnName = newColumnName;
    if (dataSet != null) {
      openDataSet();
    }
  }

  public String getColumnName() {
    return columnName;
  }

  public void addDataChangeListener(
    DataChangeListener listener) {

    dataChangeCaster.add(listener);
  }

  public void removeDataChangeListener(
    DataChangeListener listener) {

    dataChangeCaster.remove(listener);
  }

  public void addNavigationListener(
    NavigationListener listener) {

    navigationCaster.add(listener);
  }

  public void removeNavigationListener(
    NavigationListener listener) {

    navigationCaster.remove(listener);
  }

  public void dataChanged(DataChangeEvent e) {
    setSelected(null, true);
    dataChangeCaster.dispatch(e);
  }

  public void navigated(NavigationEvent e) {
    setSelected(null, true);
    navigationCaster.dispatch(e);
  }

  private void openDataSet() {
    if (dataSet != null &&
      !dataSet.isOpen()) {

      try {
        dataSet.open();
      } catch (Exception ex) {
        return;
      }
    }
  }
}