package higa.dbswing;

//import com.sun.java.swing.DefaultButtonModel;
//import com.sun.java.swing.ButtonGroup;
import javax.swing.DefaultButtonModel;
import javax.swing.ButtonGroup;

import java.awt.event.ActionEvent;

import java.io.Serializable;

import higa.sql.dataset.CheckAware;
import higa.sql.dataset.DataChangeEvent;
import higa.sql.dataset.DataChangeListener;
import higa.sql.dataset.NavigationEvent;
import higa.sql.dataset.NavigationListener;
import higa.sql.dataset.IDataSet;

public class HButtonModel extends DefaultButtonModel
  implements CheckAware,
  DataChangeListener, NavigationListener,
  Serializable {

  protected IDataSet dataSet;
  protected String columnName;
  protected String checkValue;
  protected String uncheckValue;

  public HButtonModel() {
  }

  public HButtonModel(IDataSet newDataSet,
    String newColumnName, String newCheckValue,
    String newUncheckValue) {

    setDataSet(newDataSet);
    setColumnName(newColumnName);
    setCheckValue(newCheckValue);
    setUncheckValue(newUncheckValue);
  }

  public synchronized void setDataSet(IDataSet newDataSet) {
    if (getHButtonGroup() != null) {
      return;
    }

    _setDataSet(newDataSet);
  }

  protected synchronized void _setDataSet(
    IDataSet newDataSet) {

    if (dataSet != null) {
      dataSet.removeDataChangeListener(this);
      dataSet.removeNavigationListener(this);
    }

    dataSet = newDataSet;

    if (dataSet != null) {
      dataSet.addDataChangeListener(this);
      dataSet.addNavigationListener(this);
    }
  }

  public IDataSet getDataSet() {
    HButtonGroup hbg = getHButtonGroup();
    if (hbg != null) {
      return hbg.getDataSet();
    } else {
      return dataSet;
    }
  }

  public synchronized void setColumnName(
    String newColumnName) {
    
    if (getHButtonGroup() != null) {
      return;
    }

    columnName = newColumnName;
  }

  public String getColumnName() {
    HButtonGroup hbg = getHButtonGroup();
    if (hbg != null) {
      return hbg.getColumnName();
    } else {
      return columnName;
    }
  }

  public synchronized void setCheckValue(
    String newCheckValue) {

    checkValue = newCheckValue;
    if (isSelected()) {
      super.setSelected(false);
    }
  }

  public String getCheckValue() {
    return checkValue;
  }

  public synchronized void setUncheckValue(
    String newUncheckValue) {

    uncheckValue = newUncheckValue;
  }

  public String getUncheckValue() {
    return uncheckValue;
  }

  public HButtonGroup getHButtonGroup() {
    if (group == null || !(group instanceof HButtonGroup)) {
      return null;
    } else {
      return (HButtonGroup)group;
    }
  }

  public void setPressed(boolean b) {
    if ((isPressed() == b) || !isEnabled()) {
      return;
    }

    if (b == false && isArmed()) {
      setSelected(!this.isSelected());
    }

    if (b) {
      stateMask |= PRESSED;
    } else {
      stateMask &= ~PRESSED;
    }

    fireStateChanged();
    if(!isPressed() && isArmed()) {
      fireActionPerformed(new ActionEvent(
        this, ActionEvent.ACTION_PERFORMED,
        getActionCommand()));
    }
  }

  public void setSelected(boolean newSelected) {
    IDataSet ds = getDataSet();
    String cn = getColumnName();
    if (isSelected() == newSelected ||
      ds == null || !ds.isOpen() ||
      ds.getColumn(cn) == null) {

      return;
    }

    try {
      if (newSelected) {
        ds.set(cn, checkValue);
      } else {
        ds.set(cn, uncheckValue);
      }
    } catch (Exception ex) {
      ex.printStackTrace();
      return;
    }

    super.setSelected(newSelected);
  }

  public void setGroup(ButtonGroup newButtonGroup) {
    HButtonGroup hbg = getHButtonGroup();
    if (hbg != null) {
      hbg.removeDataChangeListener(this);
      hbg.removeNavigationListener(this);
    }

    super.setGroup(newButtonGroup);
    hbg = getHButtonGroup();

    if (hbg != null) {
      _setDataSet(null);
      hbg.addDataChangeListener(this);
      hbg.addNavigationListener(this);
    }
  }

  public ButtonGroup getGroup() {
    return group;
  }

  public void dataChanged(DataChangeEvent e) {
    updateState();
  }

  public void navigated(NavigationEvent e) {
    updateState();
  }

  void updateState() {
    IDataSet ds = getDataSet();
    String cn = getColumnName();
    if (ds == null || ds.getColumn(cn) == null) {
      return;
    }

    String value = ds.getString(cn);
    if ((value == null || value.length() == 0) &&
      (checkValue == null || checkValue.length() == 0) ||
      (value != null && value.equals(checkValue))) {

      super.setSelected(true);
    } else {
      super.setSelected(false);
    }
  }
}