package higa.jhcl.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Point;

import java.sql.Types;

import java.text.Format;

import java.io.Serializable;

import higa.jhcl.model.ItemPainter;
import higa.jhcl.model.ItemEditor;
import higa.jhcl.model.ItemEditSite;

import higa.util.Formatter;

public class ColumnView implements Cloneable,
  ItemEditSite, Serializable {

  private String name;
  private Font font;
  private Insets margins;
  private int alignment;
  private Color background;
  private Color foreground;
  private String caption;
  private int width;
  private int type = Types.OTHER;
  private ItemPainter itemPainter;
  private ItemEditor itemEditor;
  private Point editClickPoint;
  private Format formatter;
  private String formatPattern;

  public ColumnView() {}

  public ColumnView(ColumnView cv) {
    if (cv == null) {
      return;
    }
    
    name = cv.name;
    font = cv.font;
    alignment = cv.alignment;
    background = cv.background;
    foreground = cv.foreground;
    caption = cv.caption;
    width = cv.width;
    type = cv.type;
    itemPainter = cv.itemPainter;
    itemEditor = cv.itemEditor;
    formatter = cv.formatter;
    formatPattern = cv.formatPattern;
  }

  public void merge(ColumnView cv) {
    if (cv == null) {
      return;
    }
    
    if (name == null) {
      setName(cv.name);
    }

    if (font == null) {
      setFont(cv.font);
    }

    if (alignment == 0) {
      setAlignment(cv.alignment);
    }

    if (background == null) {
      setBackground(cv.background);
    }

    if (foreground == null) {
      setForeground(cv.foreground);
    }

    if (caption == null) {
      setCaption(cv.caption);
    }

    if (width == 0) {
      setWidth(cv.width);
    }

    if (type == Types.OTHER) {
      setType(cv.type);
    }

    if (itemPainter == null) {
      setPainter(cv.itemPainter);
    }

    if (itemEditor == null) {
      setEditor(cv.itemEditor);
    }

    if (formatter == null) {
      setFormatter(cv.formatter);
    }

    if (formatPattern == null) {
      setFormatPattern(cv.formatPattern);
    }
  }

  public Object clone() {
    ColumnView clone = new ColumnView(this);
    return clone;
  }

  public void setName(String newName) {
    name = newName;
    if (caption == null) {
      caption = name;
    }
  }

  public String getName() {
    return name;
  }

  public void setFont(Font newFont) {
    font = newFont;
  }

  public Font getFont() {
    return font;
  }

  public void setItemMargins(Insets newMargins) {
    margins = newMargins;
  }

  public Insets getItemMargins() {
    return margins;
  }

  public void setAlignment(int newAlignment) {
    alignment = newAlignment;
  }

  public int getAlignment() {
    return alignment;
  }

  public void setBackground(Color newColor) {
    background = newColor;
  }

  public Color getBackground() {
    return background;
  }

  public void setForeground(Color newColor) {
    foreground = newColor;
  }

  public Color getForeground() {
    return foreground;
  }

  public void setCaption(String newCaption) {
    caption = newCaption;
  }

  public String getCaption() {
    if (caption == null || caption.length() == 0) {
      return name;
    } else {
      return caption;
    }
  }

  public void setWidth(int newWidth) {
    width = newWidth;
  }

  public int getWidth() {
    return width;
  }

  public void setType(int newType) {
    type = newType;
  }

  public int getType() {
    return type;
  }

  public void setPainter(ItemPainter itemPainter) {
    this.itemPainter = itemPainter;
  }

  public ItemPainter getPainter() {
    return itemPainter;
  }

  public void setEditor(ItemEditor itemEditor) {
    this.itemEditor = itemEditor;
  }

  public ItemEditor getEditor() {
    return itemEditor;
  }

  public void safeEndEdit(boolean post) {}

  public void setEditClickPoint(Point point) {
    editClickPoint = point;
  }

  public Point getEditClickPoint() {
    return editClickPoint;
  }

  public void setFormatter(Format formatter) {
    formatPattern = null;
    _setFormatter(formatter);
  }

  public void _setFormatter(Format formatter) {
    this.formatter = formatter;
    if (formatter == null) {
      return;
    }

    itemPainter = new FocusableItemPainter(
      new SelectableItemPainter(
      new TextItemPainter(formatter)));

    itemEditor = new TextItemEditor(formatter);
  }

  public Format getFormatter() {
    return formatter;
  }

  public void setFormatPattern(String pattern) {
    formatPattern = pattern;
    if (pattern == null || pattern.length() == 0) {
      return;
    }
    
    formatter = Formatter.getDefaultFormatter(type, pattern);
    _setFormatter(formatter);
  }

  public String getFormatPattern() {
    return formatPattern;
  }
}