package higa.jhcl.view;

import java.awt.Container;
import java.awt.SystemColor;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Insets;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.AWTEvent;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.FocusEvent;

import higa.jhcl.model.ItemPainter;
import higa.jhcl.model.ItemPaintSite;

import higa.util.Alignment;
import higa.util.ActionMulticaster;

public class HComponent
  extends Container
  implements ItemPaintSite {

  private boolean readOnly;
  private boolean focusAware = true;
  private int state;
  private int alignment =
    Alignment.CENTER | Alignment.MIDDLE;

  private ActionMulticaster actionCaster =
    new ActionMulticaster();

  public HComponent() {
    setBackground(SystemColor.control);
    enableEvents(AWTEvent.FOCUS_EVENT_MASK |
      AWTEvent.KEY_EVENT_MASK |
      AWTEvent.MOUSE_EVENT_MASK |
      AWTEvent.MOUSE_MOTION_EVENT_MASK);
  }

  public Insets getItemMargins() {
    return new Insets(0,0,0,0);
  }

  public void setAlignment(int alignment) {
    this.alignment = alignment;
  }

  public int getAlignment() {
    return alignment;
  }

  public void setSelected(boolean selected) {
    if (selected) {
      this.state |= ItemPainter.SELECTED;
    } else {
      this.state &= ~ItemPainter.SELECTED;
    }

    repaint(100);
  }

  public boolean isSelected() {
    return (state & ItemPainter.SELECTED) != 0;
  }

  public void setEnabled(boolean enable) {
    if (isEnabled() != enable) {
      if (enable) {
        state &= ~ItemPainter.DISABLED;
      } else {
        state |= ItemPainter.DISABLED;
        state &= ~ItemPainter.SELECTED;
        state &= ~ItemPainter.ROLLOVER;
      }

      super.setEnabled(enable);
      repaint(100);
    }
  }

  public void setReadOnly(boolean ro) {
    readOnly = ro;
  }

  public boolean isReadOnly() {
    return readOnly;
  }

  public int getState() {
    return state;
  }

  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  public void update(Graphics g) {
    paint(g);
  }

  public void setVisible(boolean visible) {
    if (!visible) {
      setSelected(false);
    }

    super.setVisible(visible);
  }

  public void addActionListener(
    ActionListener listener) {

    actionCaster.add(listener);
  }

  public void removeActionListener(
    ActionListener listener) {

    actionCaster.remove(listener);
  }

  protected void setFocusAware(boolean aware) {
    focusAware = aware;
  }

  protected boolean isFocusAware() {
    return focusAware;
  }

  protected Rectangle getClipBounds() {
    Graphics g = getGraphics();
    if (g == null) {
      return null;
    }

    Dimension vp = getSize();
    Rectangle c = g.getClipBounds();

    Rectangle clip;
    if (c == null) {
      clip = new Rectangle(0, 0, vp.width, vp.height);
    } else if (c.width > vp.width || c.height > vp.height) {
      clip = c.intersection(
      new Rectangle(0, 0, vp.width, vp.height));
    } else {
      clip = c;
    }

    if (!clip.isEmpty()) {
      return clip;
    } else {
      return null;
    }
  }

  protected void processEvent(AWTEvent e) {
    if (e instanceof ActionEvent) {
      processActionEvent((ActionEvent)e);
    } else {
      super.processEvent(e);
    }
  }

  protected void processActionEvent(ActionEvent e) {
    if (actionCaster.hasListeners()) {
      actionCaster.dispatch(e);
    }
  }

  protected void processKeyEvent(KeyEvent e) {
    switch (e.getID()) {
      case KeyEvent.KEY_PRESSED:
        processKeyPressed(e);
        break;
      case KeyEvent.KEY_TYPED:
        processKeyTyped(e);
        break;
      case KeyEvent.KEY_RELEASED:
        processKeyReleased(e);
        break;
    }

    super.processKeyEvent(e);
  }

  protected void processKeyPressed(KeyEvent e) {
    state |= ItemPainter.SELECTED;
  }

  protected void processKeyTyped(KeyEvent e) {}

  protected void processKeyReleased(KeyEvent e) {
    state &= ~ItemPainter.SELECTED;
  }

  protected void processMouseEvent(MouseEvent e) {
    switch (e.getID()) {
      case MouseEvent.MOUSE_PRESSED:
        processMousePressed(e);
        break;
      case MouseEvent.MOUSE_RELEASED:
        processMouseReleased(e);
        break;
      case MouseEvent.MOUSE_CLICKED:
        processMouseClicked(e);
        break;
      case MouseEvent.MOUSE_ENTERED:
        processMouseEntered(e);
        break;
      case MouseEvent.MOUSE_EXITED:
        processMouseExited(e);
        break;
    }

    super.processMouseEvent(e);
  }

  protected void processMouseMotionEvent(MouseEvent e) {
    switch (e.getID()) {
      case MouseEvent.MOUSE_MOVED:
        processMouseMoved(e);
        break;
      case MouseEvent.MOUSE_DRAGGED:
        processMouseDragged(e);
        break;
    }

    super.processMouseMotionEvent(e);
  }

  protected void processMousePressed(MouseEvent e) {
    state |= ItemPainter.SELECTED;
    if (isFocusTraversable()) {
      requestFocus();
    }
  }

  protected void processMouseReleased(MouseEvent e) {
    state &= ~ItemPainter.SELECTED;
  }

  protected void processMouseClicked(MouseEvent e) {}

  protected void processMouseEntered(MouseEvent e) {
    state |= ItemPainter.ROLLOVER;
  }

  protected void processMouseExited(MouseEvent e) {
    state &= ~ItemPainter.ROLLOVER;
    state &= ~ItemPainter.SELECTED;
  }

  protected void processMouseMoved(MouseEvent e) {}

  protected void processMouseDragged(MouseEvent e) {}

  protected void processFocusEvent(FocusEvent e) {
    if (focusAware) {
      switch (e.getID()) {
        case FocusEvent.FOCUS_GAINED:
          processFocusGained(e);
          break;
        case FocusEvent.FOCUS_LOST:
          processFocusLost(e);
          break;
      }

      repaint();
    }

    super.processFocusEvent(e);
  }

  protected void processFocusGained(FocusEvent e) {
    state |= ItemPainter.FOCUSED;
  }

  protected void processFocusLost(FocusEvent e) {
    state &= ~ItemPainter.FOCUSED;
    state &= ~ItemPainter.SELECTED;
  }
}