package higa.jhcl.view;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import java.awt.Color;
import java.awt.SystemColor;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Font;
import java.awt.Component;

import higa.jhcl.model.ItemPainter;
import higa.jhcl.model.ItemPaintSite;

public class SelectableItemPainter
  implements ItemPainter, ItemPaintSite, Serializable {

  private transient ItemPainter painter;
  private transient ItemPaintSite site;
  private transient int state;

  private Color selectedFg =
    SystemColor.textHighlightText;

  private Color selectedBg =
    SystemColor.textHighlight;

  private Color disabledFg =
    SystemColor.menuText;

  private Color disabledBg =
    SystemColor.menu;

  public SelectableItemPainter() {}

  public SelectableItemPainter(ItemPainter painter) {
    this.painter = painter;
  }

  public void setPainter(ItemPainter painter) {
    this.painter = painter;
  }

  public ItemPainter getPainter() {
    return painter;
  }

  public void setSelectedBackground(Color c) {
    selectedBg = c;
  }

  public Color getSelectedBackground() {
    return selectedBg;
  }

  public void setSelectedForeground(Color c) {
    selectedFg = c;
  }

  public Color getSelectedForeground() {
    return selectedFg;
  }

  public void setDisabledBackground(Color c) {
    disabledBg = c;
  }

  public Color getDisabledBackground() {
    return disabledBg;
  }

  public void setDisabledForeground(Color c) {
    disabledFg = c;
  }

  public Color getDisabledForeground() {
    return disabledFg;
  }

  public Dimension getPreferredSize(
    Object data, Graphics g,
    int state, ItemPaintSite site) {

    if (painter != null) {
      return painter.getPreferredSize(
        data, g, state, this);
    } else {
      return new Dimension();
    }
  }

  public void paint(Object data, Graphics g,
    Rectangle rect, int state, ItemPaintSite site) {

    this.state = state;
    this.site = site;
    if (painter != null) {
      painter.paint(data, g, rect, state, this);
    }
  }

  public Color getBackground() {
    Color c = null;
    if ((state & SELECTED) != 0) {
      if ((state & DISABLED) != 0) {
        c = disabledBg;
      } else {
        c = selectedBg;
      }
    } else if (site != null) {
      c = site.getBackground();
    }

    return c;
  }

  public Color getForeground() {
    return (state & SELECTED) != 0 ?
      ((state & DISABLED) != 0 ?
      disabledFg : selectedFg) :
      site != null ? site.getForeground() : null;
  }

  public Font getFont() {
    return site != null ? site.getFont() : null;
  }

  public int getAlignment() {
    return site != null ? site.getAlignment() : 0;
  }

  public Insets getItemMargins() {
    return site != null ? site.getItemMargins() : null;
  }

  private void writeObject(ObjectOutputStream s)
    throws IOException {

    s.defaultWriteObject();
    s.writeObject(
      painter instanceof Serializable ? painter : null);
  }

  private void readObject(ObjectInputStream s)
    throws IOException, ClassNotFoundException {
    
    s.defaultReadObject();
    Object data = s.readObject();
    if (data instanceof ItemPainter) {
      painter = (ItemPainter)data;
    }
  }
}