package higa.sql.dataset;

import java.io.Serializable;

import higa.jhcl.model.MatrixModel;
import higa.jhcl.model.MatrixModelEvent;
import higa.jhcl.model.MatrixModelListener;
import higa.jhcl.model.MatrixModelMulticaster;
import higa.jhcl.model.MatrixSubfocusEvent;
import higa.jhcl.model.MatrixSubfocusListener;
import higa.jhcl.model.MatrixLocation;

import higa.util.VetoException;

public class MatrixDataSetManager
  implements MatrixModel, DataChangeListener,
  MatrixSubfocusListener, Serializable {

  private IDataSet dataSet;
  private boolean eventEnabled = true;
  private MatrixModelMulticaster modelCaster =
    new MatrixModelMulticaster();

  public MatrixDataSetManager() {
  }

  public MatrixDataSetManager(IDataSet ds) {
    setDataSet(ds);
  }

  public void setDataSet(IDataSet ds) {
    if (dataSet != null) {
      dataSet.removeDataChangeListener(this);
    }

    dataSet = ds;

    if (dataSet != null) {
      dataSet.addDataChangeListener(this);
    }
  }

  public IDataSet getDataSet() {
    return dataSet;
  }

  public boolean canSet(int rowIndex, int columnIndex) {
    if (rowIndex >= 0 &&
      rowIndex < dataSet.getRowCount() &&
      columnIndex >= 0 &&
      columnIndex < dataSet.getColumnCount()) {

      return true;
    } else {
      return false;
    }
  }

  public void set(int rowIndex, int columnIndex, Object data) {
    if (!canSet(rowIndex, columnIndex)) {
      return;
    }
    
    try {
      dataSet.set(rowIndex, columnIndex, data);
      processModelEvent(new MatrixModelEvent(
        this, MatrixModelEvent.ITEM_CHANGED,
        new MatrixLocation(rowIndex, columnIndex)));
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void addRow() {
    int rowCount = dataSet.getRowCount();
    addRow(rowCount);
  }

  public void addRow(int rowIndex) {
    if (rowIndex < 0 || rowIndex > getRowCount()) {
      return;
    }
    
    try {
      dataSet.insertRow(rowIndex);
      processModelEvent(new MatrixModelEvent(
        this, MatrixModelEvent.ROW_ADDED,
        new MatrixLocation(rowIndex, 0)));
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void removeRow(int rowIndex) {
    if (rowIndex < 0 || rowIndex >= getRowCount()) {
      return;
    }

    try {
      dataSet.deleteRow(rowIndex);
      processModelEvent(new MatrixModelEvent(
        this, MatrixModelEvent.ROW_REMOVED,
        new MatrixLocation(rowIndex, 0)));
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void removeAllRows() {
    try {
      dataSet.last();
      int rowCount = dataSet.getRowCount();
      dataSet.first();
      for (int i = 0; i < rowCount; i++) {
        dataSet.deleteRow();
      }
      
      processModelEvent(new MatrixModelEvent(
        this, MatrixModelEvent.STRUCTURE_CHANGED));
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void addColumn() {
    return;
  }

  public void addColumn(int columnIndex) {
    return;
  }

  public void removeColumn(int columnIndex) {
    return;
  }

  public void removeAllColumns() {
    return;
  }

  public void setEventEnabled(boolean enable) {
    eventEnabled = enable;
  }

  public Object get(int rowIndex, int columnIndex) {
    return dataSet.get(rowIndex, columnIndex);
  }

  public int getRowCount() {
    return dataSet.getRowCount();
  }

  public int getColumnCount() {
    return dataSet.getColumnCount();
  }

  public void addModelListener(MatrixModelListener listener) {
    modelCaster.add(listener);
  }

  public void removeModelListener(MatrixModelListener listener) {
    modelCaster.remove(listener);
  }

  public void dataChanged(DataChangeEvent e) {
    int rowIndex = dataSet.getRowIndex();
    switch (e.getStatus()) {
      case Status.STRUCTURE_CHANGED:
        processModelEvent(new MatrixModelEvent(
          this, MatrixModelEvent.STRUCTURE_CHANGED));

        break;
      case Status.INSERTED:
        processModelEvent(new MatrixModelEvent(
          this, MatrixModelEvent.ROW_ADDED,
          new MatrixLocation(rowIndex, 0)));

        break;
      case Status.UPDATED:
        processModelEvent(new MatrixModelEvent(
          this, MatrixModelEvent.ROW_CHANGED,
          new MatrixLocation(rowIndex, 0)));

        break;
      case Status.DELETED:
        processModelEvent(new MatrixModelEvent(
        this, MatrixModelEvent.ROW_REMOVED,
        new MatrixLocation(rowIndex, 0)));

        break;
      default:
    }
  }

  public void subfocusChanging(MatrixSubfocusEvent e)
    throws VetoException {}

  public void subfocusChanged(MatrixSubfocusEvent e) {
    int rowIndex = e.getLocation().rowIndex;
    if (rowIndex != dataSet.getRowIndex()) {

      try {
        dataSet.goToRow(rowIndex);
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }

  protected void processModelEvent(MatrixModelEvent e) {
    if (eventEnabled && modelCaster.hasListeners()) {
      modelCaster.dispatch(e);
    }
  }
}